;********************************************************************
;                         Tomasz Jaboski                          *
;                 Mikrokontrolery PIC16F8x w praktyce               *
;                      Wydawnictwo BTC, www.btc.pl                  *
;********************************************************************

;odczytanie zawartoci pamici ROM ukadu DS1990A
;zawarto pamici wyswietlana jest na wywietlaczu LCD
;w formacie hex. Jako pierwsza wywietlana jest wyliczina CRC 
;Przy prawidlowym odczycie CRC powinna by zerowa. Jeeli nie jest 
;rowna zero, to odczytane dane nie mog by uznane za prawidlowe
;podaczenie wywietlacza do moduu AVT pokazane jest na rys.8.19
;linia DS podczona jest do linii PORTA2 
#include 16f84.h
dana_DS		equ 0ch
licznik		equ 0dh
z_bit		equ 0eh
del		equ 0fh		;zmienna procedur opoznienia
TMP		equ 10h
TMP1		equ 11h
msek		equ 12h
licznik1	equ 13h
CRC		equ 14h

#define E	porta,1
#define RS	porta,0

#define DS	porta,2
#define presen	z_bit,1
#define crc_ok	z_bit,0

		__config 	h'3ff1'
		org 0

		bsf RP0		;bank1
		movlw .4
		movwf trisa	;linia RA2 wejciowa
		clrf trisb	;wszystkie linie wyjciowe
		bcf RP0
		call iniLCD
		movlw 80h	;adres 0 wywietlacza LCD
		call wrcmd
		call _czytaj_DS	;odczytanie zawartoci ROM DS1990A
		
		movf CRC,w	;wywietlenie obliczonej CRC
		call hex_ascii
		movlw .7	;pozostae bajty do wywietlenia
		movwf licznik1
		movlw 20h
		movwf fsr
pppp		movf indf,w
		call hex_ascii
		incf fsr,f
		decfsz licznik1,f
		goto pppp
		
		sleep
		

;********************************************************************
;procedura odczytania zawartoci pamici ROM DS1990A.Odczytane bajty 
; w pamici RAM od adresu 20h. Jezeli CRC OK to bit crc_ok=1
;********************************************************************
_czytaj_DS	call res_ds	;impuls inicjalizacji
		btfsc presen
		goto et_odb_DS1
		movlw .250	;odczekanie 0.5sek
		call delay_msek
		movlw .250
		call delay_msek
		goto _czytaj_DS	;czekaj na impuls presence

et_odb_DS1	movlw 33h	;kod odczytu pamici ROM
		call wr_ds

		movlw 20h
		movwf fsr	;pocztek bufora RAM
		clrf CRC	;warto pocztkowa CRC
		movlw .8	;licznik odbieranych bajtw
		movwf licznik1

et_odb_DS	call rd_ds	;odczytanie bajtu z DS1990A
		movwf indf	;zapisywanie do bufora RAM
		incf fsr,f
		call wyl_CRC
		decfsz licznik1,f
		goto et_odb_DS
		
		bcf crc_ok	;crc_ok=0		
		movf CRC,w
		btfsc Z		
		bsf crc_ok	;crc_ok=1 kiedy CRC=0
		return


;********************************************************************
;procedura wykonywania sekwencji inicjalizacyjnej
;********************************************************************
res_ds		bcf porta,2	;RA2=0
		bsf RP0		;bank1
		bcf DS		;zero do bitu TRISB - poczatek impulsu reset
		call del_480	;odczekanie ok 480usek
		bsf RP0
		bsf DS		;koniec impulsu reset
		bcf RP0		;bank0

		call del_60	;odczekanie 60usek
		bsf presen
		btfsc DS	;sprawdzanie stanu linii
		bcf presen
		
		call del_405	;odczekanie 405usek
		return
;********************************************************************
;procedura czytania z magistrali 1wire, przeczytana dana w rejestrze
;W i dana_DS
;********************************************************************
rd_ds		movlw .8
		movwf licznik
		
et_rd		bcf porta,2	;zerowanie RA2
		bsf RP0		;bank1
		bcf DS		;na lini zero
		nop		;opoznienie 6usek dla fxtal=2MHz
		nop
		nop
		bsf DS		;na linie jedynka
		bcf RP0		;bank0
		
		nop		;opoznienie 10usek dla fxtal=2MHz
		nop
		nop
		nop
		nop
		;bit danych do C
		bcf C		;C=0
		btfsc DS	;pomi C=1 kiedy DS=0
		bsf C		;C=1
		
		rrf dana_DS,f
		call del_46	;oponienie 46usek
		decfsz licznik,f
		goto et_rd
		movf dana_DS,w
		return	
;********************************************************************
;procedura zapisywania danych na magistral 1wire
;dana zapisywana musi by umieszczona w rejestrze W
;********************************************************************
wr_ds		movwf dana_DS	;zachowanie danej do wysania
		movlw .8	;licznik bitw
		movwf licznik
		bcf porta,2

et_wrd		bsf RP0		;bank1
		rrf dana_DS,f	;pierwszy wysany LBS
		bcf DS		;na lni zero
		btfss C
		call del_60	;opnienie 60usek dla zera
		nop		;6usek dla zapisu jedynki
		nop
		nop
		bsf RP0		;bank1
		bsf DS		;na lini jedynka
		btfsc C
		call del_60	;czekaj 60usek dla jedynki
		decfsz licznik,f
		goto et_wrd
		return
		
;********************************************************************
;procedura wyliczania wielomianu kontrolnego CRC, wielomian umieszczony
;w zmiennej CRC, a dana w rejestrze dana_DS
;********************************************************************
wyl_CRC		movlw .8
		movwf licznik
		movf dana_DS,w
		movwf TMP

et_crc0		rrf TMP,f
		;bc et_crc3
		btfsc C
		goto et_crc3
		btfsc CRC,7
		goto et_crc4
et_crc1		bcf C
		rlf CRC,f
et_crc2		decfsz licznik,f
		goto et_crc0
		return

et_crc3		btfsc CRC,7
		goto et_crc1
et_crc4		movlw 18h
		xorwf CRC,f
		bsf C
		rlf CRC,f
		goto et_crc2
		


;--------------------------------------------------------------------
;procedura zapisu danej (RS=1) lub instrukcji (RS=0)
;zapisywana osimiobitowa dana znajduje si w rejestrze W
;--------------------------------------------------------------------
wrdata		bsf RS
		goto et_wr
wrcmd		bcf RS		;RS=0 rejestr instrukcji IR
et_wr		movwf TMP	;instrukcja do TMP
		bsf E		;E=1
		swapf TMP,f	;zamiana polbajtow

		movlw 0f0h	;jedynki na starszym polbajcie
		iorwf TMP,w	;f0h.OR.TMP->W
		movwf portb	;na portb starszy polbajt
		bcf E		;E=0
		nop
		nop
		bsf E		;E=1

		swapf TMP,w	;ponowna zamiana polbajtow i wpisnie do W
		iorlw 0f0h	;f0h.OR.W->W
		movwf portb
		bcf E		;E=0
		btfsc RS	;jaki jest stan linii RS
		goto et_wrdata	;zapis danej
		movlw .2
		call delay_msek	;opznienie 2msek dla instrukcji
		return
et_wrdata	call del_100	;opoznienie 100usek dla kodow znakow
		return
	
;--------------------------------------------------------------------
;porcedura inicjalizacji - interfejs 4bitowy
;--------------------------------------------------------------------
iniLCD		movlw .50	;opoznienie 50msek
		call delay_msek
		
		bcf RS		;RS=0 rejestr IR
		bsf E		;E=1
		movlw .3
		movwf portb

		bcf E		;E=0
		movlw .5	;opoznienie 5msek
		call delay_msek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek

		movlw 28h	;interfejs 4bitowy, 2linie znaki 5x7
		call wrcmd		
		movlw .8	;display off
		call wrcmd
		movlw .1	;display clear
		call wrcmd
		movlw .6	;entry mod
		call wrcmd
		movlw 0ch	;display on
		call wrcmd	
		return
del_480		movlw .240
		movwf del
		goto del_
				
del_405		movlw .202
		movwf del
		goto del_

del_60		movlw .30
		movwf del
		goto del_

del_46		movlw .23
		movwf del
		goto del_
;--------------------------------------------------------------------
;opnienie ok 100usek
;--------------------------------------------------------------------
del_100		movlw .50
		movwf del
del_		bsf RP0		;bank1	
		movlw .8	;option_reg=8 preskaler do WDT 
		movwf option_reg
		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del100	movf del,w	;50->W (50*2usek=100usek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del100	;nie rowne
		return

		
;--------------------------------------------------------------------
;opnienie w milisekundach
;--------------------------------------------------------------------		
delay_msek	movwf msek
		bsf RP0		;bank1	
		movlw .1	;option_reg=1 preskaler do TMR0 
		movwf option_reg;podzal preskalera 1:4

et_del1		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del2		movlw .125	;125->W (125*4*2usek=1msek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del2	;nie rowne

		decfsz msek,f	;modyfikacja licznika milisekund
		goto et_del1
		return

hex_ascii	movwf TMP1
		swapf TMP1,w
		andlw 0fh
		call hexascii
		call wrdata
		movf TMP1,w
		andlw 0fh
		call hexascii
		call wrdata
		return
hexascii	
		movwf TMP
		movlw .10		
		subwf TMP,w
		btfsc C
		goto _litera
		movf TMP,w
		addlw 30h
		return
_litera		movf TMP,w
		addlw 37h
		return
		
		end

