;********************************************************************
;                         Tomasz Jaboski                          *
;                 Mikrokontrolery PIC16F8x w praktyce               *
;                      Wydawnictwo BTC, www.btc.pl                  *
;********************************************************************

;program wysya cig znakw ASCII "www.microchip.com" z prdkoci 
;transmisji 1200Bd  dla Fxtal=2MHz. Po wysaniu wchodzi w ptl 
;odbioru znakw (nadawanych z prdkoci  1200Bd). Odbierane znaki s 
;wywietlane na wywietlaczu LCD - schemat pocze rysunek 8.24
#include 16f84.h
msek		equ 10h		;licznik milisekund do opnie
TMP		equ 11h
adres		equ 12h
l_bitow		equ 13h		;licznik wysyanych i odbieranych bitw
#define E	porta,1
#define RS	porta,0
#define TxD	porta,3		;linia TxD
#define RxD	porta,2		;linia RxD

;********************************************************************
		__config 	h'3ff1'
		org 0
		bsf RP0		;bank 1
		movlw .4
		movwf trisa 	;RA2 wejciowa (RxD)
		movlw 0f0h
		movwf trisb
		bcf RP0	
		call iniLCD
		movlw 80h	;adres 00
		call wrcmd
		call wys_microchip
eto		call rs_rxd
		call wrdata
		movlw .200
		call delay_msek
		goto eto				
		sleep

;--------------------------------------------------------------------
;procedura wysyania napisu koczcego si znakiem koca (00h)
;kody znakw umieszczone s w tablicy kodow (dt......)
;--------------------------------------------------------------------
wys_microchip	movlw .0	;adres poczatku w tablicy kodow
		movwf adres	
et_wys		call napis	;pobranie kodu z tablicy
		addlw 0		;sprawdzenie czy kod=0
		btfsc Z
		return		;kod=0 (kod konca)
		call rs_txd	;wpisanie kodu do pamieci DD RAM
		incf adres,f	;modyfikacja adresu tablicy kodow (retlw)
		movf adres,w
		goto et_wys		

napis		addwf pcl,f
		dt "www.microchip.com",0
;--------------------------------------------------------------------
;procedura odbierania bajtu przez interfejs RS232
;8bitw informacyjnych bez bitu parzystoci, 1bit stopu
;odebrany bajt w rejestrze W
;--------------------------------------------------------------------
rs_rxd		movlw .8 	;liczba bitw iformacyjnych
		movwf l_bitow
		btfsc RxD	;czekanie na bit startu
		goto rs_rxd
		
		call del_hb	;opnienie 1/2 bitu

rs_rxd1		call del_hb	;opnienie 1bitu
		call del_hb
		bcf C		;zeruj C
		btfsc RxD
		bsf C
		rrf TMP,f	;przesu C do TMP
		decfsz l_bitow,f
		goto rs_rxd1
		movf TMP,w	;odebrana dana do W
		return
		
		
;--------------------------------------------------------------------
;procedura wysyania bajtu przez interfejs RS232C
;prdko transmisji 1200Bd 8bitw informacyjnych, bez bitu pzrystoci
;2 bity stopu. Wysyany bajt umieszczony jest w rejestrze W
;--------------------------------------------------------------------
rs_txd		movwf TMP	;przesanie W do TMP
		movlw .9	;8 bitw informacyjnych plus bit startu
		movwf l_bitow
		bsf C		;bit stopu do C
		bcf TxD		;bit startu

rs_txd1		call del_hb	;opnienie 1bitu
		call del_hb
		rrf TMP,f	;do C bit danych
		btfsc C	
		bsf TxD		;bit danych = 1
		btfss C
		bcf TxD		;bit danych = 0 
		decfsz l_bitow,f;modyfikacja liczby wyslanych bitow
		goto rs_txd1
		call del_hb	;czeknie na dwa bity stopu
		call del_hb
		call del_hb
		call del_hb
		return

;--------------------------------------------------------------------
;opnienie 1/2bitu dla prdkoci 1200Bd dla Fxtal 2MHz
;--------------------------------------------------------------------
del_hb		bsf RP0		;bank1	
		movlw .8		;option_reg=8 preskaler do WDT 
		movwf option_reg
		bcf RP0		;bank0
		clrf tmr0		;zerowanie licznika TMR0

et_hb		movlw .188		;208->W (208*2usek=416usek)
		subwf tmr0,w		;czy W = tmr0
		btfss C
		goto et_hb		;nie rowne
		return
;--------------------------------------------------------------------
;--------------------------------------------------------------------
;procedura zapisu danej (RS=1) lub instrukcji (RS=0)
;zapisywana osimiobitowa dana znajduje si w rejestrze W
;--------------------------------------------------------------------
wrdata		bsf RS
		goto et_wr
wrcmd		bcf RS		;RS=0 rejestr instrukcji IR
et_wr		movwf TMP	;instrukcja do TMP
		bsf E		;E=1
		swapf TMP,f	;zamiana polbajtow

		movlw 0f0h	;jedynki na starszym polbajcie
		iorwf TMP,w	;f0h.OR.TMP->W
		movwf portb	;na portb starszy polbajt
		bcf E		;E=0
		nop
		nop
		bsf E		;E=1

		swapf TMP,w	;ponowna zamiana polbajtow i wpisnie do W
		iorlw 0f0h	;f0h.OR.W->W
		movwf portb
		bcf E		;E=0
		btfsc RS	;jaki jest stan linii RS
		goto et_wrdata	;zapis danej
		movlw .2
		call delay_msek	;opznienie 2msek dla instrukcji
		return
et_wrdata	call del_100	;opoznienie 100usek dla kodow znakow
		return
	
;--------------------------------------------------------------------
;porcedura inicjalizacji - interfejs 4bitowy
;--------------------------------------------------------------------
iniLCD		movlw .50	;opoznienie 50msek
		call delay_msek
		
		bcf RS		;RS=0 rejestr IR
		bsf E		;E=1
		movlw .3
		movwf portb

		bcf E		;E=0
		movlw .5	;opoznienie 5msek
		call delay_msek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek
		bsf E		;E=1
		nop		;ponowne wpisanie 3 modulu
		nop
		bcf E		;E=0
		call del_100	;czekaj 100usek

		movlw 28h	;interfejs 4bitowy, 2linie znaki 5x7
		call wrcmd		
		movlw .8	;display off
		call wrcmd
		movlw .1	;display clear
		call wrcmd
		movlw .6	;entry mod
		call wrcmd
		movlw 0ch	;display on
		call wrcmd	
		return
;--------------------------------------------------------------------
;opnienie ok 100msek Fxtal 2MHz
;--------------------------------------------------------------------
del_100		bsf RP0		;bank1	
		movlw .8	;option_reg=8 preskaler do WDT 
		movwf option_reg
		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del100	movlw .50	;50->W (50*2usek=100usek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del100	;nie rowne
		return

		
;--------------------------------------------------------------------
;opnienie w milisekundach Fxtal 2MHz
;--------------------------------------------------------------------		
delay_msek	movwf msek
		bsf RP0		;bank1	
		movlw .1	;option_reg=1 preskaler do TMR0 
		movwf option_reg;podzal preskalera 1:4

et_del1		bcf RP0		;bank0
		clrf tmr0	;zerowanie licznika TMR0

et_del2		movlw .125	;125->W (125*4*2usek=1msek)
		bcf RP0		;bank0
		subwf tmr0,w	;czy W = tmr0
		btfss C
		goto et_del2	;nie rowne

		decfsz msek,f	;modyfikacja licznika milisekund
		goto et_del1
		return

		end	
	end
